#!/usr/bin/perl -w

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.

use taskman_lib;
use strict;
use CGI;

#--------------------------------------------------------------------------
# date_select($date)
#
# Returns a HTML form object for selecting a date.  $date should be
# the number of seconds since the epoch for the default date.  If
# $date is omitted, today's date will be used.

sub
date_select
{
    my $date = shift @_;
    my ( $i, $day, $month, $year, @tmp, $output, $optstr, @d, @m, @y );
    unless (defined $date) {
	$date = time();
    }
    @tmp   = localtime($date);
    $day   = $tmp[3];
    $month = $tmp[4]+1;
    $year  = $tmp[5]+1900;
    @d     = (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31);
    @m     = (1,2,3,4,5,6,7,8,9,10,11,12);
    @y     = (2000,2001,2002,2003,2004,2005,2006,2007);
    $output = "<table border=1><tr><td><select name=day>";
    $output .= optionlist ( $day, @d );
    $output .= "</select></td><td><select name=month>";
    $output .= optionlist ( $month, @m );
    $output .= "</select></td><td><select name=year>";
    $output .= optionlist ( $year, @y );
    $output .= "</select></td></table>";
    return $output;
}
    


#--------------------------------------------------------------------------
# main

my $dbh   = dbserver_connect('agnostos');
my $query = new CGI;
my $id    = $query -> param('todoid');
my $title = ""; 
my $note  = ""; 
my $desc  = "";
my ($dbq, $priority, $status, $option, $optstr, $i);
my ($userid, $groupid, $depid, $tmp, $tmpid, %tmpopts, $action, $date);
my @options = ('done','queued','wait','in-progress','cancelled');
 
my $tableref = get_cookie($query);


if (defined $id) {
    print html_header("Edit task",$query);
    print "<h2>Edit task</h2>\n";
}
else {
    print html_header("Add new task",$query);
    print "<h2>Add task</h2>\n";
}
# Select existing data

if (defined $id) {
    $dbq = $dbh -> prepare("SELECT title, description, status, priority, note, UNIX_TIMESTAMP(date)
                            FROM todo
                            WHERE todoid = $id");
    $dbq -> execute;
    ($title, $desc, $status, $priority, $note, $date) = $dbq -> fetchrow_array;
    $desc = "" unless (defined $desc);
    $note = "" unless (defined $note);
    $dbq -> finish;
    $dbq = $dbh -> prepare ("SELECT userid, groupid, depid FROM assigned WHERE todoid = $id");
    $dbq -> execute;
    ($userid, $groupid, $depid) = $dbq -> fetchrow_array;
    $dbq -> finish;
}
else {
    $status='wait';
    $priority='10';
}

print "Note- priorities are ordered from 1 (highest) to 10 (lowest)\n";
print "<form action=update_task.pl method=post>\n";

if (defined $id) {
    $action = 'update';
    print "<input type=hidden name=todoid value=$id>\n";
}
else {
    $action = 'add';
    $date = time();
}
print "<input type=hidden name=action value=$action>\n";
print "<table><tr><td>Task:<td><input type=text value=\"$title\" name=title size=80>\n</table>";
print "<table><tr><td>Status:<td><select name=status>\n";
print optionlist( $status, @options );
print "</select>\n";
print "<td>Priority:<td><select name=priority>\n";
print optionlist($priority, '1', '2', '3', '4', '5', '6', '7', '8', '9', '10');
print "</select>\n";
print "<td>Due Date:<td>".date_select($date);
if ( (! defined $date) or $action eq 'add') {
    print "<td>None<td><input type=checkbox name=nodate checked value=true>";
}
else {
    print "<td>None<td><input type=checkbox name=nodate value=true>";
}
print "</table><table>\n";

# User Select Box
print "<tr><td>User:<td><select name=userid>\n";
$dbq = $dbh -> prepare("SELECT userid, username FROM user");
$dbq -> execute;
while ( ($tmpid, $tmp) = $dbq -> fetchrow_array ){
    $tmpopts{$tmpid} = $tmp;
}
$dbq -> finish;
if ($action eq 'add') {
    if (defined $$tableref{'user'})  {
	$userid = $$tableref{'user'};
    }
    else {
	$userid = 1;
    }
}   
print named_optionlist( $userid, %tmpopts );
print "</select>\n";
undef %tmpopts;

# Group Select Box
print "<td>Workgroup:<td><select name=groupid>\n";
$dbq = $dbh -> prepare("SELECT groupid, groupname FROM workgroup");
$dbq -> execute;
while ( ($tmpid, $tmp) = $dbq -> fetchrow_array ){
    $tmpopts{$tmpid} = $tmp;
}
$dbq -> finish;
if ($action eq 'add') {
    if (defined $$tableref{'workgroup'})  {
	$groupid = $$tableref{'workgroup'};
    }
    else {
	$groupid = 1;
    }
}   

print named_optionlist( $groupid, %tmpopts );
print "</select>\n";
undef %tmpopts;

# Department Select Box
print "<td>Department:<td><select name=depid>\n";
$dbq = $dbh -> prepare("SELECT depid, depname FROM department");
$dbq -> execute;
while ( ($tmpid, $tmp) = $dbq -> fetchrow_array ){
    $tmpopts{$tmpid} = $tmp;
}
$dbq -> finish;
if ($action eq 'add') {
    if (defined $$tableref{'department'})  {
	$depid = $$tableref{'department'};
    }
    else {
	$depid = 1;
    }
}   
print named_optionlist( $depid, %tmpopts );
print "</select>\n";
print "</table>\n";

print "<table>\n";
print "<tr><td>Description:<td><textarea name=desc rows=5 cols=80 wrap=physical>$desc</textarea>\n";
print "<tr><td>Note:<td><textarea name=note rows=10 cols=80 wrap=physical>$note</textarea>\n";
print "<tr><td><input type=submit value=\"Update\"><td> </table>\n";
print html_footer;


$dbq -> finish;
$dbh -> disconnect;


