package taskman_lib;

# Copyright (C) 2000 Julian T. J. Midgley <jtjm@xenoclast.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# A copy of version 2 of the GNU General Public License may be found
# in the file "LICENCE" in the original Agnostos installation driectory.

# Provides task manager specific libraries

use strict;
use vars qw(@ISA @EXPORT);
use DBI;
use Sys::Hostname;


use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
	     html_header
	     html_footer
	     get_table
	     gen_assigned_name
	     optionlist
	     named_optionlist
	     sql_quote
	     fetch_ents
	     dbserver_connect
	     Do_SQL_Query
	     get_cookie
	     set_cookie
	    );

#--------------------------------------------------------------------------
# get_cookie ($query)

sub
get_cookie 
{
    my $query = shift @_;
    my (%tablehash, @tables, $table, $cookie_string);
    
    $cookie_string = $query -> cookie ("AGNOSTOS_TABLEREF");
    unless (defined $cookie_string) {
	$tablehash{'fubar'}='completely';
	return \%tablehash;
    }
    @tables = split (/;/,$cookie_string);
    foreach $table (@tables) {
	my @items = split (/,/,$table);
	next unless (defined $items[0]);
	$tablehash{$items[0]} = $items[1];
    }
    return \%tablehash;
}


#--------------------------------------------------------------------------
# set_cookie ($tableref, $query)

sub 
set_cookie 
{
    my ($tableref, $query) = @_;
    my ($table, $cookie_string, $cookie_out);
    $cookie_string = "";
    foreach $table (keys %$tableref) {
	$cookie_string .= "$table,$$tableref{$table};";
    }
    $cookie_out = $query -> cookie( -name    => 'AGNOSTOS_TABLEREF',
		                    -expires => '+1y',
               		            -value   => "$cookie_string" );
    return $cookie_out;
}

#--------------------------------------------------------------------------
# numeric sort
sub
numeric
{
    $a <=> $b;
}


#--------------------------------------------------------------------------
# Do_SQL_Query ($dbh, $query)


sub 
Do_SQL_Query 
{
    my $dbh = shift @_;
    my $query = shift @_ || return(0);
    my $dbq_query = $dbh -> prepare($query);
    my $num_rows = $dbq_query -> execute;
	
    return ( $dbq_query );
}


#--------------------------------------------------------------------------
# dbserver_connect( $database );
# Returns a handle to the database $database.

sub
dbserver_connect 
{
    my $database = shift @_;
    my $type     = "mysql";
    my $user     = "agnostos";
    my $host     = "localhost";
    my $dbh = DBI->connect("DBI:$type:$database:$host", "$user")
	or die "Can't connect to $database database";
    return($dbh);
}

#--------------------------------------------------------------------------
# html_header($title,$query,@cookie)
sub
html_header 
{
    my ($title, $query, $cookie) = @_;
    my $ret;
    if ( defined $cookie ) { 
	$ret = $query -> header(-cookie=>$cookie);
    }
    else {
	$ret = $query -> header();
    }
    $ret .= "\n<html><head><meta name=\"pragma\" content=\"nocache\"><title>".
	    "$title</title></head><body bgcolor=white vlink=black>\n";
    return $ret;
}

#--------------------------------------------------------------------------
# html_footer()
sub
html_footer
{
    return "</body></html>\n";
}

#--------------------------------------------------------------------------
# gen_assigned_name($table)
# returns the appropriate name of the id column in the assigned table.

sub
gen_assigned_name
{
    my $table = shift @_;
    my $name;
    if ($table eq 'user') {
	$name = 'userid';
    }
    elsif ($table eq 'workgroup') {
	$name = 'groupid';
    }
    elsif ($table eq 'department') {
	$name = 'depid';
    }
    else {
	die "Invalid table name at gen_assigned_name($table)";
    }
}


#--------------------------------------------------------------------------
# get_table($table)
# Sets the database elements to use

sub
get_table
{
    my ($id, $name);
    my $table = shift @_;
    if ($table eq 'user') {
	$id   = 'userid';
	$name = 'username';
    }
    elsif ($table eq 'workgroup') {
	$id   = 'groupid';
	$name = 'groupname';
    }
    elsif ($table eq 'department') {
	$id   = 'depid';
	$name = 'depname';
    }
    else {
	die "Invalid table name: $table";
    }
    return ($id, $name);
}

#--------------------------------------------------------------------------
# optionlist ($default, @options)
# returns an HTML option list

sub 
optionlist 
{
    my $default = shift @_;
    $default = "" unless (defined $default);
    my @options = @_;
    my ($option, $str);
    $str = "";
    foreach $option (@options) {
	$str .= "<option ";
	if ( $option eq $default ) {
	    $str .= "selected ";
	}
	$str .= "value=$option>$option\n";
    }
    return $str;
}

#--------------------------------------------------------------------------
# named_optionlist($default, %options)
# returns an HTML option list, where the values and options differ

sub 
named_optionlist 
{
    my $default = shift @_;
    $default = "" unless (defined $default);
    my %options = @_;
    my ($option, $str);
    $str = "";
    foreach $option (sort numeric keys %options) {
	$str .= "<option ";
	if ( $option eq $default ) {
	    $str .= "selected ";
	}
	$str .= "value=$option>$options{$option}\n";
    }
    return $str;
}

#--------------------------------------------------------------------------
# sql_quote($string) 
#
# Returns $string with all special characters (like
# single quotation marks), commented out

sub
sql_quote
{
    my $string = shift @_;
    $string =~ s/\'/\'\'/g;
    return $string;
}

#--------------------------------------------------------------------------
# fetch_ents($dbh, $table) 
# returns a hash of the entity ids and names, keyed by id.

sub
fetch_ents 
{
    my ($table, $dbh) = @_;
    my ($entid, $entname, %entities);
    my ($id, $name) = get_table($table);
    my $dbq = $dbh -> prepare("SELECT $id, $name FROM $table");
    $dbq -> execute;
    while ( ($entid, $entname) = $dbq -> fetchrow_array ) {
	if ($entname eq 'UNSET') {
	    $entname = "&nbsp;&nbsp;&nbsp; -- &nbsp;&nbsp;&nbsp;";
	}
	$entities{$entid} = $entname;
    }
    $dbq -> finish;
    return %entities;
}

1;
